@@ -28,232 +28,8 @@ USA.*/
 #include "general.h"
 void ErrorMessage(char *msg, int fatal);
 
-/* routines based on singular value decomposition.... They are not as well
-   structured as they could be and have been largely superceeded by the
-   more efficient tridiagonalisation based routines that follow */
-
-double svdopt(B,S,X,W,y,f,r,method,outcv) matrix B,S,X,W,y,f;double r,*outcv;int method;
-
-/* Let A be an influence matrix:
-
-   A = X inv(B+rS)X'W
-
-   and assume f contains the fitted values corresponding to the B and r
-   supplied:
-   The routine finds r to minimise either:
-   1: GCV score
-   2: OCV score
-   using a singular value decomposition algorithm.
-   B must be positive definite.
-   There is O(n^3) start up cost for this routine, but thereafter the
-   evaluations are (O(n^2)) (about 4n^2, actually ). The routine returns the
-   optimal r value.
-   X in this formulation is a design matrix. So that usually
-   B = X'WX
-
-*/
-
-{ matrix e,XU,U,E;
-  int gridpoints=25,i,ok;
-  double range=100000.0,rmin,cv,cvmin,dr,c1,c2,c3,cnew=0.0,r1,r2,r3,rnew=0.0,tau;
-  tau=2.0/(1.0+sqrt(5.0));
-
-  /* Now find the decomposition of (B+rS) that allows quick inversion */
-  U=initmat(B.r,B.r);E=initmat(B.c,1L);
-  suminvert(S,B,U,E);
-  /* The influence matrix is now: A=X U Diag(r/(r E.V[i]+1)) U'X'W/r. Accordingly
-     obtain XU..... */
-  XU=initmat(X.r,U.c);multi(2,XU,X,U,0,0,0); /* added 9/11/96 - since previous version lacked design matrix. */
-  /* form f=Ay */
-  e=initmat(y.r,1L); /* required as offset by svdoptcv */
 
 
-  r=m1norm(B)/m1norm(S);
-  freemat(U);
-  rmin=r;cvmin=svdoptcv(r,XU,E,W,e,y,method);
-  dr=exp(2.0/(gridpoints-1)*log(range));r/=range;
-  /* gridsearch for minimiser */
-  for (i=0;i<gridpoints;i++)
-  { cv=svdoptcv(r,XU,E,W,e,y,method);
-    if (cv<cvmin) { cvmin=cv;rmin=r; }
-    r*=dr;
-  }
-  /* Now polish the minimum, by golden section */
-  r3=rmin*dr;r1=rmin/dr;c2=cvmin;
-  c3=svdoptcv(r3,XU,E,W,e,y,method);
-  c1=svdoptcv(r1,XU,E,W,e,y,method);
-  if ((c2<c3)&&(c2<c1))
-  { r2=tau*(r3-r1);
-    c2=svdoptcv(r2,XU,E,W,e,y,method);
-    /* perform golden section search for smoothing parameter */
-    ok=1;
-    if (ok)
-    while ((r3-r1)>0.00001*r2)
-    { if ((r2-r1)<(r3-r2))
-      { rnew=r1+(r3-r1)*tau;
-        cnew=svdoptcv(rnew,XU,E,W,e,y,method);
-	     if (cnew<c2)
-	     { r1=r2;r2=rnew;c2=cnew;}
-	     else { r3=rnew;}
-      } else
-      { rnew=r1+(r3-r2)*(1.0-tau);
-	     cnew=svdoptcv(rnew,XU,E,W,e,y,method);
-	     if (cnew<c2)
-	     { r3=r2;r2=rnew;c2=cnew;}
-	     else { r1=rnew;}
-      }
-    }
-    rmin=rnew;cvmin=cnew;
-  }
-  r=rmin;
-  for (i=0;i<f.r;i++) e.V[i]=y.V[i]*W.V[i];
-  e=vecmult(XU,e,1);
-  for (i=0;i<f.r;i++) e.V[i]/=r*E.V[i]+1.0;
-  e=vecmult(XU,e,0);
-  for (i=0;i<f.r;i++) f.V[i]=e.V[i];
-  freemat(e);freemat(XU);freemat(E);
-  *outcv=cvmin;
-  return(rmin);
-}
-
-
-
-double svdoptcv(r,ZU,E,W,e,y,method) double r;matrix ZU,E,W,e,y;int method;
-
-/* service routine for svdopt() - calculates CV scores */
-
-{ matrix EUZ,TrA,z;
-  int i,j;
-  double tr,cv;
-  EUZ=initmat(ZU.c,ZU.r);TrA=initmat(ZU.r,1L);
-  /* get elements of trace of A given r */
-  for (i=0;i<EUZ.r;i++) for (j=0;j<EUZ.c;j++)
-  EUZ.M[i][j]=ZU.M[j][i]/(r*E.V[i]+1.0);
-  for (i=0;i<ZU.r;i++)
-  { TrA.V[i]=0.0;
-    for (j=0;j<ZU.c;j++) TrA.V[i]+=ZU.M[i][j]*EUZ.M[j][i];TrA.V[i]*=W.V[i];
-  }
-  /* Now form the residual vector */
-  z=initmat(y.r,1L);
-  for (i=0;i<y.r;i++) z.V[i]=W.V[i]*y.V[i];
-  z=vecmult(EUZ,z,0);
-  z=vecmult(ZU,z,0);
-  for (i=0;i<y.r;i++) z.V[i]=y.V[i]-e.V[i]-z.V[i];
-  cv=0.0;tr=0.0;
-  if (method==1) /* then its GCV */
-  { for (i=0;i<z.r;i++) { cv+=z.V[i]*W.V[i]*z.V[i];tr+=1.0-TrA.V[i];}
-    cv/=tr*tr;
-  } else
-  if (method==2) /* OCV */
-  for (i=0;i<z.r;i++)
-  { cv+=z.V[i]*W.V[i]*z.V[i]/((1.0-TrA.V[i])*(1.0-TrA.V[i]));}
-  freemat(z);freemat(EUZ);freemat(TrA);return(cv);
-}
-
-
-double zsvdopt(Z,B,S,X,W,f,y,r,method,outcv) matrix Z,B,S,X,W,f,y;double r,*outcv;int method;
-
-/* Let A be an influence matrix:
-
-   A = XZ inv(Z'(B+rS)Z) Z'X'W
-
-   in general f must contain the fitted values corresponding to the B and r
-   supplied, unless it is true that Ay gives the expected values according
-   to the model in which case supply f as a vector with f.r=0;
-   (this condition is not in general true for inequlity constrained problems -
-    but is true for thin plate splines subjected only to the usual equality
-    conditions).
-   The routine finds r to minimise either:
-   1: GCV score
-   2: OCV score
-   using a singular value decomposition algorithm.
-   Z'BZ must be positive definite.
-   There is O(n^3) start up cost for this routine, but thereafter the
-   evaluations are (O(n^2)) (about 4n^2, actually ). The routine returns the
-   optimal r value.
-   X in this formulation is a design matrix.
-
-*/
-
-{ matrix e,ZU,BZ,ZSZ,ZBZ,U,E;
-  int gridpoints=25,i,j,k,ok;
-  double range=100000.0,rmin,cv,cvmin,dr,c1,c2,c3,cnew=0.0,r1,r2,r3,rnew=0.0,tau;
-  tau=2.0/(1.0+sqrt(5.0));
-  /* form Z'BZ and Z'SZ these are symmetric matrices, so the op count isn't too bad */
-  BZ=initmat(B.r,Z.c);
-  for (i=0;i<BZ.r;i++) for (j=0;j<BZ.c;j++) for (k=0;k<B.c;k++)
-  BZ.M[i][j]+=B.M[i][k]*Z.M[k][j];ZBZ=initmat(Z.c,Z.c);
-  for (i=0;i<ZBZ.r;i++) for (j=0;j<=i;j++)
-  { for (k=0;k<Z.r;k++) ZBZ.M[i][j]+=Z.M[k][i]*BZ.M[k][j];
-    ZBZ.M[j][i]=ZBZ.M[j][i];
-  }
-  freemat(BZ);BZ=initmat(S.r,Z.c);
-  for (i=0;i<BZ.r;i++) for (j=0;j<BZ.c;j++) for (k=0;k<S.c;k++)
-  BZ.M[i][j]+=S.M[i][k]*Z.M[k][j];ZSZ=initmat(Z.c,Z.c);
-  for (i=0;i<ZSZ.r;i++) for (j=0;j<=i;j++)
-  { for (k=0;k<Z.r;k++) ZSZ.M[i][j]+=Z.M[k][i]*BZ.M[k][j];
-    ZSZ.M[j][i]=ZSZ.M[j][i];
-  }
-  /* Now find the decomposition of Z'(B+rS)Z that allows quick inversion */
-  U=initmat(Z.c,Z.c);E=initmat(Z.c,1L);
-  suminvert(ZSZ,ZBZ,U,E);
-  /* The influence matrix is now: A=ZU Diag(r/(r E.V[i]+1)) Z'U'W/r. Accordingly
-     obtain ZU..... */
-  ZU=initmat(X.r,U.c);multi(3,ZU,X,Z,U,0,0,0); /* added 9/11/96 - since previous version lacked design matrix. */
-  /* form e=f-Ay */
-  e=initmat(y.r,1L);
-  if (f.r) /* calculate offset for fitting problem */
-  { for (i=0;i<e.r;i++) e.V[i]=y.V[i]*W.V[i];
-    e=vecmult(ZU,e,1);
-    for (i=0;i<e.r;i++) e.V[i]/=r*E.V[i]+1.0;
-    e=vecmult(ZU,e,0);
-    for (i=0;i<e.r;i++) e.V[i]=f.V[i]-e.V[i]; /* as A changes f=e + Ay */
-  }
-  r=m1norm(ZBZ)/m1norm(ZSZ);
-  freemat(BZ);freemat(ZSZ);freemat(ZBZ);
-  freemat(U);
-  rmin=r;cvmin=svdoptcv(r,ZU,E,W,e,y,method);
-  dr=exp(2.0/(gridpoints-1)*log(range));r/=range;
-  /* gridsearch for minimiser */
-  for (i=0;i<gridpoints;i++)
-  { cv=svdoptcv(r,ZU,E,W,e,y,method);
-    if (cv<cvmin) { cvmin=cv;rmin=r; }
-    r*=dr;
-  }
-  /* Now polish the minimum, by golden section */
-  r3=rmin*dr;r1=rmin/dr;c2=cvmin;
-  c3=svdoptcv(r3,ZU,E,W,e,y,method);
-  c1=svdoptcv(r1,ZU,E,W,e,y,method);
-  if ((c2<c3)&&(c2<c1))
-  { r2=tau*(r3-r1);
-    c2=svdoptcv(r2,ZU,E,W,e,y,method);
-    /* perform golden section search for smoothing parameter */
-    ok=1;
-    if (ok)
-    while ((r3-r1)>0.00001*r2)
-    { if ((r2-r1)<(r3-r2))
-      { rnew=r1+(r3-r1)*tau;
-	     cnew=svdoptcv(rnew,ZU,E,W,e,y,method);
-	     if (cnew<c2)
-	     { r1=r2;r2=rnew;c2=cnew;}
-	     else { r3=rnew;}
-      } else
-      { rnew=r1+(r3-r2)*(1.0-tau);
-	     cnew=svdoptcv(rnew,ZU,E,W,e,y,method);
-	     if (cnew<c2)
-	     { r3=r2;r2=rnew;c2=cnew;}
-	     else { r1=rnew;}
-      }
-    }
-    rmin=rnew;cvmin=cnew;
-  }
-  freemat(e);freemat(ZU);freemat(E);  *outcv=cvmin;
-  return(rmin);
-}
-
-/* Routines based on tridiagonalisation of a transformed smoothness matrix
-   See Gu, as reported in Wahba 1990. These routines are better than the svd
-   routines, furthermore multiple smoothing parameters are dealt with. */
 
 double TrInf(matrix *X,matrix *Z,matrix *w,matrix *S,double rho)
 
